package org.apache.bcel.verifier.statics;

import org.apache.bcel.verifier.exc.LocalVariableInfoInconsistentException;
import org.apache.bcel.verifier.exc.AssertionViolatedException;
import org.apache.bcel.generic.Type;
import java.util.Arrays;

public class LocalVariablesInfo
{
    private final LocalVariableInfo[] localVariableInfos;
    
    LocalVariablesInfo(final int maxLocals) {
        Arrays.setAll(this.localVariableInfos = new LocalVariableInfo[maxLocals], i -> new LocalVariableInfo());
    }
    
    public void add(final int slot, final String name, final int startPc, final int length, final Type type) throws LocalVariableInfoInconsistentException {
        if (slot < 0 || slot >= this.localVariableInfos.length) {
            throw new AssertionViolatedException("Slot number for local variable information out of range.");
        }
        this.localVariableInfos[slot].add(name, startPc, length, type);
        if (type == Type.LONG) {
            this.localVariableInfos[slot + 1].add(name, startPc, length, LONG_Upper.theInstance());
        }
        if (type == Type.DOUBLE) {
            this.localVariableInfos[slot + 1].add(name, startPc, length, DOUBLE_Upper.theInstance());
        }
    }
    
    public LocalVariableInfo getLocalVariableInfo(final int slot) {
        if (slot < 0 || slot >= this.localVariableInfos.length) {
            throw new AssertionViolatedException("Slot number for local variable information out of range.");
        }
        return this.localVariableInfos[slot];
    }
}
